--
-- Praga
--

Praga = {};

function Praga.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Praga:load(xmlFile)
    local grainTipAnimCurve = AnimCurve:new(linearInterpolator4);
    local keyI = 0;
    self.vysun = false;   
    self.pocetBaliku = 0;
    
    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    
    self.zaloha_capacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.capacity"), 0.0);
    while true do
      local key = string.format("vehicle.grainPlane.tipkey(%d)", keyI);
      local t = getXMLFloat(xmlFile, key.."#time");
      local yValue = getXMLFloat(xmlFile, key.."#y");
      local scaleX,scaleY,scaleZ = Utils.getVectorFromString(getXMLString(xmlFile, key.."#scale"));
      if y == nil or scaleX == nil or scaleY == nil or scaleZ == nil then
        break;
      end;
      grainTipAnimCurve:addKeyframe({x=scaleX, y=scaleY, z=scaleZ, w=yValue, time = t});
      keyI = keyI +1;
    end;
    if keyI > 0 then
      self.grainTipAnimCurve = grainTipAnimCurve;
    end;
    
	  self.allowFillFromAir = false;
	  self.TipTriggerInRange = false;
    self.drawGrainLevel = SpecializationUtil.callSpecializationsFunction("drawGrainLevel");
    self.loadItems = SpecializationUtil.callSpecializationsFunction("loadItems");

    self.hudBasePoxX = 0.8325;
    self.hudBasePoxY = (1-0.99);
    self.hudBaseWidth = 0.16;
    self.hudBaseHeight = 0.1625;
    self.hudBaseOverlay = Overlay:new("hudBaseOverlay", "dataS/missions/hud_vehicle_base".. g_languageSuffix .. ".png", self.hudBasePoxX, self.hudBasePoxY, self.hudBaseWidth, self.hudBaseHeight);
    
    self.lapacBaliku = {};
    self.lapacBaliku.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lapacBaliku#index"));
    local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.lapacBaliku#minTrans"));
    self.lapacBaliku.minTrans = {};
    self.lapacBaliku.minTrans[1] = x;
    self.lapacBaliku.minTrans[2] = y;
    self.lapacBaliku.minTrans[3] = z;
        
    local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.lapacBaliku#maxTrans"));
    self.lapacBaliku.maxTrans = {};
    self.lapacBaliku.maxTrans[1] = x;
    self.lapacBaliku.maxTrans[2] = y;
    self.lapacBaliku.maxTrans[3] = z;
    
    self.lapacBaliku.animTime = getXMLInt(xmlFile, "vehicle.lapacBaliku#time");
    
    self.lapacBaliku.pp = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lapacBaliku#ppIndex"));
    self.lapacBaliku.lz = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lapacBaliku#lzIndex"));    
    local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.lapacBaliku#pocetBaliku"));
    self.lapacBaliku.pocetBaliku = x*y*z;
    self.lapacBaliku.rada_x=x;
    self.lapacBaliku.rada_y=y;
    self.lapacBaliku.rada_z=z;
    self.lapacBaliku.baliky = {};
    self.lapacBaliku.balikyNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lapacBaliku#balikyIndex"));
    local s = getXMLString(xmlFile, "vehicle.lapacBaliku#balikyIndex");    
    for i=1, self.lapacBaliku.pocetBaliku do
      self.lapacBaliku.baliky[i] = {};
      self.lapacBaliku.baliky[i].node = Utils.indexToObject(self.components,s.."|"..tostring(i-1));
      self.lapacBaliku.baliky[i].balik = nil;
      self.lapacBaliku.baliky[i].balikCol = 0; 
    end;
    self.fillNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fillNode#index"));      
end;

function Praga:delete()
    if self.hudBaseOverlay then
        self.hudBaseOverlay:delete();
    end;
end;

function Praga:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Praga:keyEvent(unicode, sym, modifier, isDown)
end;

function Praga:update(dt) 
  if self:getIsActiveForInput() then
    if (self.pocetBaliku > 0) or (self.vysun) then
      local s = "";    
      if self.vysun then
        s="Ukoncit vysun baliku";
      else
        s="Vysunout baliky";
      end;
      self.fillLevel=self.pocetBaliku;
      self.capacity=self.lapacBaliku.pocetBaliku;
      g_currentMission:addHelpButtonText(s, InputBinding.IMPLEMENT_EXTRA);
    else
      self.capacity=self.zaloha_capacity;  
    end;
    if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
      self.vysun = not self.vysun;
      if self.vysun then
        if self.tipAnimCharSet ~= 0 then
          if getAnimTrackTime(self.tipAnimCharSet, 0) < 0.0 then
            setAnimTrackTime(self.tipAnimCharSet, 0, 0.0);
          end;
          setAnimTrackSpeedScale(self.tipAnimCharSet, 0, self.tipAnimSpeedScale);
          enableAnimTrack(self.tipAnimCharSet, 0);
        end;
      else
        if self.tipAnimCharSet ~= 0 then
          if getAnimTrackTime(self.tipAnimCharSet, 0) > self.tipAnimDuration then
            setAnimTrackTime(self.tipAnimCharSet, 0, self.tipAnimDuration);
          end;
          setAnimTrackSpeedScale(self.tipAnimCharSet, 0, -self.tipAnimSpeedScale);
          enableAnimTrack(self.tipAnimCharSet, 0);
        end;   
      end;
    end;
  end;
  local x, y, z = getTranslation(self.lapacBaliku.node);
  local newtrans = {x,y,z};
  if not self.vysun or ((self.vysun) and (getAnimTrackTime(self.tipAnimCharSet, 0) > self.tipAnimDuration)) then  
    local x, y, z = getTranslation(self.lapacBaliku.node);
    local trans = {x,y,z};
    newtrans = Utils.getMovedLimitedValues(trans, self.lapacBaliku.maxTrans, self.lapacBaliku.minTrans, 3, self.lapacBaliku.animTime, dt, not self.vysun);
    setTranslation(self.lapacBaliku.node, unpack(newtrans));
  end;
  
  if newtrans ~= nil then    
    local rozsah = self.lapacBaliku.maxTrans[3] - self.lapacBaliku.minTrans[3];
    local straw = "data/maps/models/objects/strawbale/strawbaleBaler.i3d"
    local hay = "data/maps/models/objects/strawbale/haybaleBaler.i3d"    
    for i=1,self.lapacBaliku.rada_z do
      local z=self.lapacBaliku.rada_z-i+1;
      if (newtrans[3] <= (((rozsah / (self.lapacBaliku.rada_z))*(i))+self.lapacBaliku.minTrans[3]+0.1)) and (newtrans[3] >= (((rozsah / (self.lapacBaliku.rada_z))*(i))+self.lapacBaliku.minTrans[3]-0.1)) then
        for y=1,self.lapacBaliku.rada_y do
          for x=1,self.lapacBaliku.rada_x do
            local index = (y-1)*(self.lapacBaliku.rada_x*self.lapacBaliku.rada_z)+((z-1)*self.lapacBaliku.rada_x)+x;
            if self.lapacBaliku.baliky[index].balik ~= nil then
              local bx, by, bz = getWorldTranslation(self.lapacBaliku.baliky[index].balik);
              local rx,ry,rz = getWorldRotation(self.lapacBaliku.baliky[index].balik);
              setTranslation(self.lapacBaliku.baliky[index].balik, bx, by, bz);
              setRotation(self.lapacBaliku.baliky[index].balik, rx,ry,rz);
              link(getRootNode(), self.lapacBaliku.baliky[index].balik);
              setRigidBodyType(self.lapacBaliku.baliky[index].balik,"Dynamic");
              setCollisionMask(self.lapacBaliku.baliky[index].balik,self.lapacBaliku.baliky[index].balikCol);
              if g_currentMission.itemsToSave == nil then 
                g_currentMission.itemsToSave = {}; 
              end;
              child = self.lapacBaliku.baliky[index].balik;
              name = getName(child)             
              if name == "strawbaleBaler" then 
                file = straw;
              end;
              if name == "haybaleBaler" then 
                file = hay;
              end;              
              table.insert(g_currentMission.itemsToSave,{["class"]="MT",["i3dFilename"]=file,["node"]=child,["childIndex"]=0});
              self.lapacBaliku.baliky[index].balik = nil;
              self.lapacBaliku.baliky[index].balikCol = 0;
              self.pocetBaliku=self.pocetBaliku-1;
              self.fillLevel=self.pocetBaliku;
            end;
          end;
        end;
      end;
    end;  
  end;
  
  if (self.currentFillType == FruitUtil.FRUITTYPE_UNKNOWN) and (self.pocetBaliku<self.lapacBaliku.pocetBaliku) and (not self.vysun) and (newtrans[3]==self.lapacBaliku.minTrans[3]) then
    local balik;
    local index;
    balik, index = BalikInRange(self);
    if balik ~= nil then
      self.pocetBaliku = self.pocetBaliku + 1;
      setRigidBodyType(balik,"Kinematic");
      self.lapacBaliku.baliky[self.pocetBaliku].balikCol=getCollisionMask(balik);
      setCollisionMask(balik,0);
      link(self.lapacBaliku.baliky[self.pocetBaliku].node,balik);
      setTranslation(balik,0,0,0);
      setRotation(balik,0,0,0);           
      self.lapacBaliku.baliky[self.pocetBaliku].balik = balik;
      table.remove(g_currentMission.itemsToSave,index)
    end;
  end; 

  if (self.tipState == Trailer.TIPSTATE_CLOSED) and (self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN) and (self.fillLevel>0) then  
    self.TipTriggerInRange = (TipTriggerInRange(self)~=nil);    

    local fillTypeName = FruitUtil.fruitIndexToDesc[self.currentFillType].name;
    local grainPlane = self.grainPlanes[fillTypeName];
     
    local x, y, z = getTranslation(grainPlane);
    local trans = {x,y,z}; 
    local scaleX, scaleY, scaleZ , yTrans = self.grainTipAnimCurve:get(self.fillLevel/self.capacity);
    local sX, sY, sZ , yMax = self.grainAnimCurve:get(self.fillLevel/self.capacity);
    local newtrans = Utils.getMovedLimitedValues(trans,{x,yMax,z}, {x,yTrans,z}, 3, 2000, dt, self.TipTriggerInRange); 
    setTranslation(grainPlane, unpack(newtrans)); 
  end;
end;

function Praga:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
  self.fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
  self.itemsToSave = {};
  local itemkey = key..".items";
  local count = Utils.getNoNil(getXMLInt(xmlFile, itemkey.."#count"),0);
  if count ~= 0 then
    for i=1,count do
      self.itemsToSave[i] ={};
      local key = itemkey..".item"..i;
      local name = getXMLString(xmlFile, key.."#filename");      
      local parent = getXMLInt(xmlFile, key.."#parent");
      local col = getXMLInt(xmlFile, key.."#col");
      self.itemsToSave[i].name = name;
      self.itemsToSave[i].parent = parent;
      self.itemsToSave[i].col = col;
      self:loadItems(i);
    end;
  end;
  self.itemsToSave = nil;
  return BaseMission.VEHICLE_LOAD_OK;
end

function Praga:getSaveAttributesAndNodes(nodeIdent)
  local items ={};
  local straw = "data/maps/models/objects/strawbale/strawbaleBaler.i3d";
  local hay = "data/maps/models/objects/strawbale/haybaleBaler.i3d";
  for ii=1,self.pocetBaliku do
    local node = self.lapacBaliku.baliky[ii].balik;
    local name = getName(node);
    if name == "strawbaleBaler" then 
      name = straw;
    elseif name == "haybaleBaler" then 
      name = hay;
    end;
    table.insert(items,{node=node,parent=ii,name=name});
  end;
  local attributes = "";
  local i=0;
  if items ~= nil then 
    i= table.getn(items); 
  end;
  local nodes = "";
  nodes = nodes..nodeIdent..'<items count="'..i..'">';
  if i ~= 0 then
    nodes = nodes.."\n";
    for xx=1,i do
      local name = items[xx].name;      
      local id = items[xx].parent;
      local col = self.lapacBaliku.baliky[id].balikCol;
      if xx > 1 then 
        nodes = nodes.."\n" ;
      end;
      nodes = nodes..nodeIdent..'<item'..xx..' filename="'..name..'" parent="'..id..'" col="'..col..'" />'; 
    end;
  end;
  nodes = nodes.."\n";
  nodes = nodes..nodeIdent..'</items>';
  return attributes,nodes;
end 

function Praga:loadItems(id)
  local item = self.itemsToSave[id];
  local i = item.parent;
  local i3dNode = Utils.loadSharedI3DFile(item.name, baseDirectory);
  local itemNode = getChildAt(i3dNode,0);
  local attacher = self.lapacBaliku.baliky[i].node;
  local col = item.col;
  self.pocetBaliku = self.pocetBaliku + 1;
  setRigidBodyType(itemNode,"Kinematic");
  self.lapacBaliku.baliky[self.pocetBaliku].balikCol=col;
  setCollisionMask(itemNode,0);  
  self.lapacBaliku.baliky[self.pocetBaliku].balik = itemNode;
  link(attacher,itemNode);
  setTranslation(itemNode,0,0,0);
  setRotation(itemNode,0,0,0);  
  delete(i3dNode);   
end 

function Praga:draw()
--    self.hudBaseOverlay:render();
-- local trailerFillLevel, trailerCapacity = self:getAttachedTrailersFillLevelAndCapacity();
--    if trailerFillLevel ~= nil and trailerCapacity ~= nil then    
--      self:drawGrainLevel(trailerFillLevel+self.fillLevel, trailerCapacity+self.capacity, 101);
--    else
--  	  self:drawGrainLevel(self.fillLevel, self.capacity, 101);
--    end;

	if self:getIsActiveForInput() then
    local percent = 0;
    if self.capacity > 0 then
        percent = self.fillLevel/self.capacity*100;
    end;
    setTextBold(true);   
	  renderText(0.79,0.25,0.03,string.format("V3S:   %d(%d%%)", self.fillLevel, percent));
    setTextBold(false);	  
  end; 

end;

function Praga:setFillLevel(fillLevel, fillType)
    if not self:allowFillType(fillType, false) then
        return;
    end;

    self.currentFillType = fillType;
    self.fillLevel = fillLevel;
    if self.fillLevel > self.capacity then
        self.fillLevel = self.capacity;
    end;
    if self.fillLevel < 0 then
        self.fillLevel = 0;
        -- reset fill type as the trailer is empty
        Utils.setEmittingState(self.dischargeParticleSystems[self.currentFillType], false);
        self.currentFillType = FruitUtil.FRUITTYPE_UNKNOWN;
    end;
    if self.currentGrainPlane ~= nil then
        setVisibility(self.currentGrainPlane, false);
    end;
    if self.grainPlanes ~= nil and self.defaultGrainPlane ~= nil and fillType ~= FruitUtil.FRUITTYPE_UNKNOWN then
        local fillTypeName = FruitUtil.fruitIndexToDesc[fillType].name;
        local grainPlane = self.grainPlanes[fillTypeName];
        if grainPlane == nil then
            grainPlane = self.defaultGrainPlane;
        end;
        local yTranslation;
        if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN or self.tipState == Trailer.TIPSTATE_CLOSING then
          if self.grainTipAnimCurve then
            local scaleX, scaleY, scaleZ , yTrans = self.grainTipAnimCurve:get(self.fillLevel/self.capacity);
            yTranslation = yTrans;
            setScale(grainPlane, scaleX, scaleY, scaleZ);
          else
            local m = (self.grainPlaneMaxY - self.grainPlaneMinY) / self.capacity;
            yTranslation = m*self.fillLevel + self.grainPlaneMinY;
          end;
        else 
          if self.grainAnimCurve then
            local scaleX, scaleY, scaleZ , yTrans = self.grainAnimCurve:get(self.fillLevel/self.capacity);
            yTranslation = yTrans;
            setScale(grainPlane, scaleX, scaleY, scaleZ);
          else
            local m = (self.grainPlaneMaxY - self.grainPlaneMinY) / self.capacity;
            yTranslation = m*self.fillLevel + self.grainPlaneMinY;
          end;
        end;
        local xPos, yPos, zPos = getTranslation(grainPlane);
        setTranslation(grainPlane, xPos, yTranslation, zPos);
        setVisibility(grainPlane, self.fillLevel > 0);
        self.currentGrainPlane = grainPlane;
    end;
end;

function Praga:drawGrainLevel(level, capacity, warnPercent)
    local percent = 0;
    if capacity > 0 then
        percent = level/capacity*100;
    end;
    setTextBold(true);

    if percent >= warnPercent then
        setTextColor(1.0, 0.0, 0.0, 1.0);
    else
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;

--    renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.006, 0.03, string.format("%d(%d%%)", level, percent));
    if percent >= warnPercent then
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;
    setTextBold(false);
end;

function TipTriggerInRange(self)
	local nearestObject
	local itemNode
	local index 
	local nearestDistance = 3;
	local objectCopy = self.tipReferencePoint;	
	local px, py, pz = getWorldTranslation(objectCopy);
	for i=1, table.getn(g_currentMission.tipTriggers) do
	  itemNode = g_currentMission.tipTriggers[i].triggerId;
 	  local vx, vy, vz = getWorldTranslation(itemNode);
  	local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
    if distance < nearestDistance then
  		index = i
 	  	nearestObject = itemNode
  	  nearestDistance = distance
		end;		
	end;
  return nearestObject;
end;

function BalikInRange(self)
	local nearestObject
	local itemNode
	local index 
	local nearestDistance = 15;
	local px, py, pz = getWorldTranslation(self.lapacBaliku.pp);
	for i=1, table.getn(g_currentMission.itemsToSave) do
	  itemNode = g_currentMission.itemsToSave[i].node;
    if getParent(g_currentMission.itemsToSave[i].node) == getRootNode() then
	  	if getUserAttribute(itemNode, "isStrawbale") or getUserAttribute(itemNode, "isHaybale") then	  
 	      local vx, vy, vz = getWorldTranslation(itemNode);
  	    local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
        if distance < nearestDistance then
          local lx, ly, lz = getWorldTranslation(self.lapacBaliku.lz);
  		    if (((vx>=lx) and (vx<=px)) or ((vx<=lx) and (vx>=px))) and (((vz>=lz) and (vz<=pz)) or ((vz<=lz) and (vz>=pz))) and (vy>=ly) and (vy<=py) then
            index = i;
 	  	      nearestObject = itemNode;
  	        nearestDistance = distance;
  	        break;
		      end;
        end;
      end;
    end;      		
	end;
  return nearestObject, index;
end;
